package com.hungraim.ltc.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.pojo.entity.disable.*;
import com.hungraim.ltc.pojo.vo.disable.*;
import com.hungraim.ltc.pojo.vo.governance.ProgramListVo;
import com.hungraim.ltc.util.Result;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 失能申请Service
 *
 * @author czz
 */
@Service
public interface DisableService {


    Page<DisableVO> disableList(Page<DisableVO> page, String organId, String districtProvincial, String districtCity,
                                String districtArea, String realName, String certiCode, String dataFrom,
                                String srvOrganName, Integer applyMethod);

    /**
     * 失能审核信息详情
     *
     * @param applyId 失能人员id
     * @return 失能审核信息详情
     */
    DisableExamineInfoVO disableExamineInfo(String applyId);

    /**
     * 失能人员信息详情（失能人员信息修改申请详情页面）
     *
     * @param disabInfoId 失能人员id
     * @return 失能信息详情
     */
    DisableUpdateDetailVO disableInfoUpdateDetail(String disabInfoId);

    Result addDisable(ChDisableApply chDisableApply);

    void disableExamine(String applyId, String checkName, Short applyStatus, String checkRemark);

    /**
     * 省市区
     *
     * @param districtLevel 区域级别:1省2地级市3县
     * @param districtCode  上级区域代码
     */
    List<ChFndDistrict> district(String districtLevel, String districtCode);

    List<ChFndOrgan> fndOrganRoot(long organId);

    /**
     * 查询所有服务方式
     */
    List<SrvModeVo> cfgSrvModeList();

    /**
     * 查询机构下所有护理机构
     *
     * @param organId 机构id
     * @param isInvolveChild 是否包含子机构
     */
    List<ChSrvOrgan> srvOrganList(String organId,Long srvOrganId, Boolean isInvolveChild);

    /**
     * 查询机构意向服务机构
     *
     * @param organId 机构id
     * @param isInvolveChild 是否包含子机构
     */
    List<ChSrvOrgan> srvOrganList2(String organId, Boolean isInvolveChild);

    /**
     * 查询疾病类型
     *
     * @param parDiseTypeId 父疾病类型id
     */
    List<ChFndDiseaseType> fndDiseaseList(String parDiseTypeId);

    /**
     * 查询护理机构下所有子机构id
     *
     * @param organId 机构id
     * @return 机构及所有子机构id, 以逗号分隔, 如果没有子机构则只返回机构id
     */
    String subsidiariesOrganId(String organId);

    void updateDisable(ChDisableApply chDisableApply);

    /**
     * 获取所有省市区街道数据
     *
     * @return 返回树形结构省市区数据
     */
    List<DistrictTreeVo> allDistrict();

    /**
     * 获取所有省市区街道数据，以树形结构返回
     *
     * @return 返回树形结构省市区数据
     */
    List<DistrictTreeVo> getAllDistrictTreeData();

    /**
     * 查询所有疾病数据
     */
    List<ChFndDiseaseType> fndAllDisease();

    DisableInfoVO getDisableInfo(String disableInfoId);

    /**
     * 查询所有适用机构
     *
     * @return 返回列表
     */
    List<ChFndOrgan> fndAllOrgan(long organId);

    /**
     * 失能人员信息变更表-新增
     *
     * @param disableUpdateDetailVO
     */
    Result addOrUpdateDisInfoChange(DisableUpdateDetailVO disableUpdateDetailVO);

    /**
     * 失能人员信息变更-审核
     */
    void DisableInfoChangeExamine(String changeId, String checkName, Short applyStatus, String checkRemark);

    /**
     * 失能人员信息变更-审核列表
     */
    Page<DisableInfoChangeVo> ListDisInfoChangeExamine(Page<DisableInfoChangeVo> page,
                                                       Long organId, Boolean isChildGroup, String districtProvincial, String districtCity, String districtArea,
                                                       String realName, String certiCode, String aplOrganName, Short applyStatus);

    /**
     * 失能人员信息变更-查看修改
     *
     * @return
     */
    IPage<SearchDisableSimpleVo> searchByRealName(Integer pageNum, Integer pageSize, String realName);

    /**
     * 失能人员变更信息详情(失能人员信息修改申请详情页面-查看、修改)
     *
     * @param changeId 变更id
     * @return 失能人员变更信息详情
     */
    DisableUpdateDetailVO changeDisableDetail(Integer changeId);


    /**
     * 个人照护方案列表查询
     *
     * @param page          分页参数
     * @param srvOrganId    服务机构id
     * @return 返回个人照护方案列表
     */
    Page<DisableInfoListVo> getDisableInfoList(Page<DisableInfoListVo> page , Long srvOrganId,String certiCode,String realName,String svrModelId);


    /**
     *@author zwl
     * @param organId
     * @param districtProvincial
     * @param districtCity
     * @param districtArea
     * @param realName
     * @param certiCode
     * @param dataFrom
     * @param applyMethod
     *@return org.apache.poi.ss.usermodel.Workbook
     **/
    Workbook disableDownload(String organId, String districtProvincial, String districtCity,
                             String districtArea, String realName, String certiCode, String dataFrom,
                             Integer applyMethod);


    int disableInfoExit(Integer disableInfoId,Integer exitReasonId,Integer applyId,Integer srvOrganId,Integer programId,String exitTime);

    List<ChDisableExit> getDisableExitHis(Integer disableInfoId);

}
