package com.hungraim.ltc.governance.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.governance.dao.ChSrvSendMapper;
import com.hungraim.ltc.governance.service.*;
import com.hungraim.ltc.pojo.entity.account.ChPunchDetailsVO;
import com.hungraim.ltc.pojo.entity.disable.ChDisableInfo;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrgan;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvSend;
import com.hungraim.ltc.pojo.vo.disable.ChSrvTaskDetailVo;
import com.hungraim.ltc.pojo.vo.disable.ChrSrvVo;
import com.hungraim.ltc.pojo.vo.disable.SearchDisableSimpleVo;
import com.hungraim.ltc.pojo.vo.governance.*;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author jiaguokai
 */
@RestController
@RequestMapping("/api.governance/buildBed")
@Slf4j
public class BuildBedController {
    @Resource
    private IBuildBedService buildBedService;
    @Resource
    private SrvSendService sendService;
    @Resource
    private CrvService crvService;
    @Autowired
    private DisableInfoServie disableInfoServie;
    @Autowired
    private IRemoveBedService removeBedService;

    @Autowired
    private ChSrvSendMapper chSrvSendMapper;


    /**
     * 建床管理列表
     */
    @PostMapping("/searchOrgBed")
    public Result searchOrgBed(@RequestBody BuildBedSearchReq req) {
        return buildBedService.searchOrgBed(req);
    }

    /**
     * 建床管理新增
     */
    @PostMapping("/addOrUpdateBuildBed")
    public Result<Boolean> buildBed(BuildBedDetailVo buildBedDetailVo) {
        if (StringUtils.isBlank(buildBedDetailVo.getRealName())) {
            //姓名
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(buildBedDetailVo.getGender())) {
            //性别
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(buildBedDetailVo.getCertiCode())) {
            //身份证号
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(buildBedDetailVo.getAdressDetail())) {
            //家庭住址
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (buildBedDetailVo.getNursingDemandLevel() == null || buildBedDetailVo.getNursingDemandLevel() < 1 || buildBedDetailVo.getNursingDemandLevel() > 5) {
            //护理需求等级
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(buildBedDetailVo.getChanghuNo())) {
            //长护号
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (buildBedDetailVo.getGuaranteeMode() == null || buildBedDetailVo.getGuaranteeMode() < 1 || buildBedDetailVo.getGuaranteeMode() > 3) {
            //保障方式
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        buildBedService.saveOrUpdateBuildBed(buildBedDetailVo);
        return Result.success();
    }

    /**
     * 撤床新增或修改
     */
    @PostMapping("/addOrUpdateRemoveBed")
    public Result<Boolean> addOrUpdateRemoveBed(RemoveBedDetailVo removeBedDetailVo) {
        if (StringUtils.isBlank(removeBedDetailVo.getRealName())) {
            //姓名
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(removeBedDetailVo.getGender())) {
            //性别
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (removeBedDetailVo.getRating() == null || removeBedDetailVo.getRating() < 1 || removeBedDetailVo.getRating() > 5) {
            //护理需求等级
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(removeBedDetailVo.getChanghuNo())) {
            //长护号
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (StringUtils.isBlank(removeBedDetailVo.getTel()) || StringUtils.isBlank(removeBedDetailVo.getSpecialZone()) || StringUtils.isBlank(removeBedDetailVo.getBedNumber())  ||removeBedDetailVo.getBedDays() == null  ||removeBedDetailVo.getRemoveBedReason() == null  ||removeBedDetailVo.getBuildBedTime() == null ||removeBedDetailVo.getRemoveBedTime() == null  || StringUtils.isBlank(removeBedDetailVo.getDiagnosis()) || StringUtils.isBlank(removeBedDetailVo.getBedConstruction()) || StringUtils.isBlank(removeBedDetailVo.getNursingProcess()) || StringUtils.isBlank(removeBedDetailVo.getRemoveBedProposal())) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (removeBedDetailVo.getGuaranteeMode() == null || removeBedDetailVo.getGuaranteeMode() < 1 || removeBedDetailVo.getGuaranteeMode() > 3) {
            //保障方式
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }

        removeBedService.saveOrUpdateRemoveBed(removeBedDetailVo);
        return Result.success();
    }

    /**
     * 查询建床详情
     *
     * @param buildBedId 建床详情id
     * @return 返回建床详情
     */
    @GetMapping("getBuildBedDetail")
    public Result<BuildBedDetailVo> getBuildBedDetail(Integer buildBedId) {
        BuildBedDetailVo buildBedDetailVo = buildBedService.getBuildBedDetail(buildBedId);
        return Result.success(buildBedDetailVo);
    }


    /**
     * 查询撤床详情
     *
     * @param buildBedId 建床详情id
     * @return 返回撤床详情
     */
    @GetMapping("getRemoveBedDetail")
    public Result<RemoveBedDetailVo> getRemoveBedDetail(Integer buildBedId) {
        RemoveBedDetailVo removeBedDetailVo = removeBedService.getRemoveBedDetail(buildBedId);
        return Result.success(removeBedDetailVo);
    }

    /**
     * 功能描述: 服务分配信息检索条件查询
     */
    @PostMapping("/pageFuwuCondition")
    public Result pageFuwuCondition(ChDisableInfo disableInfo, ChSrvOrgan chSrvOrgan, ChSrvSend chSrvSend) {

        // 创建一个page对象
        IPage<SrvSendVO> page = sendService.selectPage(disableInfo, chSrvOrgan, chSrvSend);
        // 总记录数
        long total = page.getTotal();
        List<SrvSendVO> records = page.getRecords();
        Map map = new HashMap<>();
        map.put("total", total);
        map.put("records", records);
        return Result.success(map);
    }

    /**
     * 根据id查询失能人员的评定信息
     */
    @PostMapping("/queryEvaluateById")
    public Result queryEvaluateById(SrvSendVO srvSendVO) {
        Integer disableInfoId = srvSendVO.getDisableInfoId();
        Integer srvOrganId = srvSendVO.getSrvOrganId();
        if (StrUtil.isBlankIfStr(disableInfoId)) {
            return Result.failed("失能人员ID 为空！");
        }
        DisableInfoTaskVO disableInfoTaskVO = sendService.selectTaskById(disableInfoId, srvOrganId);
        List<DisableInfoTaskVO> disableInfoTaskVO1 = sendService.selectBydisableInfoId(disableInfoId);
        Map map = new HashMap<>();
        map.put("disableInfoTaskVO", disableInfoTaskVO);
        map.put("chAprTasks", disableInfoTaskVO1);
        return Result.success(map);
    }


    /**
     * 护理机构及服务方式檢索
     */
    @GetMapping("/PageCrv")
    public Result PageCrv(HttpServletRequest request, ChSrvOrgan chSrvOrgan) {
        // 调用方法实现分页查询
        IPage<ChSrvOrgan> page = crvService.seletSrvOrganPage(chSrvOrgan);
        // 总记录数
        long total = page.getTotal();
        List<ChSrvOrgan> records = page.getRecords();
        Map map = new HashMap<>();
        map.put("total", total);
        map.put("recors", records);
        return Result.success(map);
    }

    /**
     * 已分配护理机构信息查询
     */
    @PostMapping("/querySrvYet")
    public Result querySrvYet(DisableInfoVo disableInfoVo) {
        if (StrUtil.isBlankIfStr(disableInfoVo.getDisableInfoId())) {
            return Result.failed("失能人员id不能为空");
        }
        Integer status = disableInfoVo.getStatus();
        Integer sendId = disableInfoVo.getSrvSendId();
        Page<DisableInfoVo> disableInfoVos = disableInfoServie.querySrvYet(disableInfoVo.getDisableInfoId(), disableInfoVo.getCurrent(), disableInfoVo.getLimit(), status,sendId);
        long total = disableInfoVos.getTotal();
        List<DisableInfoVo> records = disableInfoVos.getRecords();
        Map map = new HashMap<>();
        map.put("total", total);
        map.put("recors", records);
        return Result.success(map);
    }

    /**
     * 提交分配机构
     */
    @PostMapping("/submit")
    public Result submit(ChrSrvVo chrSrvVo) {
        List<ChSrvSend> chSrvSends = chrSrvVo.getChSrvSends();
        if (chSrvSends != null && chSrvSends.size() > 0) {
            Long disabInfoId = null;
            for (ChSrvSend chSrvSend : chSrvSends) {
                disabInfoId = chSrvSend.getDisabInfoId();
                // 判斷參數是否存在
                if (chSrvSend.getDisabInfoId() == null || chSrvSend.getSrvOrganId() == null || chSrvSend.getSrvModeId() == null) {
                    return Result.failed("参数不全！");
                }
                List<ChSrvSend> chSrvSendsList = chSrvSendMapper.selectList(new LambdaQueryWrapper<ChSrvSend>()
                        .eq(ChSrvSend::getDisabInfoId, disabInfoId)
                        .eq(ChSrvSend::getStatus, (short) 1));
                if (chSrvSendsList.size() > 0) {
                    return Result.failed("已分配护理机构,请先终止或退出再操作");
                }

                chSrvSend.setAllotTime(new Date());
                chSrvSend.setStatus(Short.valueOf("1"));
                if (chrSrvVo.getSrvSendId() == null) {
                    Long id = sendService.selectMaxId();
                    if (id == null) {
                        id = 0L;
                    }
                    chSrvSend.setSrvSendId(++id);
                }else {
                    chSrvSend.setSrvSendId(chrSrvVo.getSrvSendId());
                }
                sendService.saveOrUpdate(chSrvSend);
            }
        } else {
            return Result.failed("提交失败，参数为空");
        }
        return Result.success();

    }

    /**
     * 终止录入
     */
    @PostMapping("/saveStopTime")
    public Result<Boolean> saveStopTime(SaveStopTime saveStopTime){
        ChSrvSend chSrvSend =  sendService.getById(saveStopTime.getSrvSendId());
        if(chSrvSend == null){
            return Result.failed("提交失败，服务分配记录不存在");
        }
        chSrvSend.setStopTime(saveStopTime.getStopTime());
        chSrvSend.setStatus((short)2);
        sendService.updateById(chSrvSend);
        return Result.success(true);
    }

    /**
     * 失能人员信息修改申请详情页搜索失能人员接口
     *
     * @param pageNum  翻页参数
     * @param pageSize 翻页参数
     * @param realName 失能人员姓名
     * @return 返回一组人员信息
     */
    @GetMapping("/buildBedSearchByRealName")
    public Result<IPage<SearchDisableSimpleVo>> buildBedSearchByRealName(@RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize, Integer srvOrganId, String realName) {
        if (null == srvOrganId) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        IPage<SearchDisableSimpleVo> searchDisableSimpleVoIPage = sendService.buildBedSearchByRealName(pageNum, pageSize, srvOrganId, realName);
        return Result.success(searchDisableSimpleVoIPage);
    }

    /**
     * 打卡查询
     * @param
     * @return
     */
    @GetMapping("/chSrvTaskDetailQuery")
    public Result  chSrvTaskDetailQuery(ChSrvTaskDetailVo vo) throws ParseException {
        Page<ChSrvTaskDetailVo> page = new Page<>(vo.getPageNum(), vo.getPageSize());
        page = buildBedService.chSrvTaskDetailQuery(page,vo);
        if(page.getSize()>0){
            return Result.success(page);
        }else {
            return Result.failed(ResultCode.SRV_ORGAN_NOT_QUERY);
        }
    }

    /**
     * 打卡详情
     * @param
     * @return
     */
    @GetMapping("/chPunchDetailsQuery")
    public Result  chPunchDetailsQuery(Long taskDetailId,String sevStarTime,String sevEndTime){
        ChPunchDetailsVO chPunchDetailsVO = buildBedService.chPunchDetailsQuery(taskDetailId,sevStarTime,sevEndTime);
        return Result.success(chPunchDetailsVO);

    }

    /**
     *  删除打卡数据
     * @param taskDetailId
     * @param userName
     * @return
     */
    @PostMapping("/chSrvTaskDetailDelete")
    public Result  chSrvTaskDetailDelete(Long taskDetailId,String userName){
        int srv = buildBedService.chSrvTaskDetailDelete(taskDetailId,userName);
        if(srv>0){
            return Result.success();
        }else {
            return Result.failed("删除失败");
        }
    }
}

