package com.hungraim.ltc.governance.controller;

import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.governance.service.SrvProgramService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvProgram;
import com.hungraim.ltc.pojo.vo.governance.ProgramDetailVo;
import com.hungraim.ltc.pojo.vo.governance.ProgramListVo;
import com.hungraim.ltc.pojo.vo.governance.ProgramSetAndProjectVo;
import com.hungraim.ltc.pojo.vo.governance.SvrSendListVo;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import com.hungraim.ltc.util.TokenParsingUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/api.governance/srvProgram")
@Slf4j
public class SrvProgramController {

    @Autowired
    private SrvProgramService srvProgramService;

    /**
     * 获取方案详情
     *
     * @param programId
     * @return
     */
    @RequestMapping("/{programId}")
    public Result getSrvProgram(@PathVariable("programId") Long programId) {

        return Result.success();
    }

    /**
     * 个人照护方案查询列表
     *
     * @param srvOrganId    服务机构id
     * @param realName      失能人员姓名
     * @param realTel       失能人员手机号
     * @param programStatus 服务方案状态
     * @return 返回列表
     */
    @RequestMapping("/programList")
    public Result getProgramList(@RequestHeader("Authorization") String token, String realName, String realTel, String programStatus, Integer pageNum, Integer pageSize) {
        long srvOrganId = TokenParsingUtil.TokenParsing(token, "srvOrganId");
        if (pageNum == null || pageNum < 1) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (pageSize == null || pageSize < 1) {
            pageSize = 10;
        }
        Page<SvrSendListVo> page = new Page<>(pageNum, pageSize);
        page = srvProgramService.getProgramList(page, srvOrganId, realName, realTel, programStatus);
        return Result.success(page);
    }

    /**
     * 个人照护方案列表查询
     *
     * @param pageNum     页码
     * @param pageSize    每页大小
     * @param disabInfoId 失能人员id
     * @param srvOrganId  服务机构id
     * @return 返回个人照护方案列表
     */
    @RequestMapping("/programSetList")
    public Result getProgramSetList(Integer pageNum, Integer pageSize, Long disabInfoId, Long srvOrganId,String status) {
        if (pageNum == null || pageNum < 1 || disabInfoId == null || disabInfoId == 0 || srvOrganId == null || srvOrganId == 0) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        if (pageSize == null || pageSize < 1) {
            pageSize = 10;
        }
        Page<ProgramListVo> page = new Page<>(pageNum, pageSize);
        page = srvProgramService.getProgramSetList(page, disabInfoId, srvOrganId, status);
        List<ProgramListVo> records = page.getRecords();
        records =records.stream().collect(
                Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparingLong(ProgramListVo::getProgramId))), ArrayList::new));
        page.setRecords(records);
        return Result.success(page);
    }

    /**
     * 查询服务计划附件列表
     *
     * @param programId 服务计划id
     */
    @RequestMapping("/programFileList")
    public Result<List<ChFndAttach>> reportFileList(Long programId) {
        Result<List<ChFndAttach>> listResult = srvProgramService.programFileList(programId);
        if (!listResult.getCode().equals(ResultCode.SUCCESS.getCode())) {
            return Result.failed("请求获取附件列表错误：错误码{" + listResult.getCode() + "},错误信息:{" + listResult.getMsg() + "}");
        }
        return listResult;
    }

    /**
     * 查看服务方案详情
     *
     * @param programId 方案id
     * @return 返回服务方案详情
     */
    @RequestMapping("/programDetail")
    public Result getProgramDetail(Long programId) {
        if (programId == null || programId == 0) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        ProgramDetailVo programDetail = srvProgramService.getProgramDetail(programId);
        return Result.success(programDetail);
    }

    /**
     * 添加服务方案时需要展示的相关信息
     *
     * @param disabInfoId 失能人员id
     * @param srvOrganId  服务机构id
     * @return 返回添加服务方案需要展示的信息
     */
    @RequestMapping("/programDetailWithAdd")
    public Result getProgramDetailWithAdd(Long disabInfoId, Long srvOrganId) {
        if (disabInfoId == null || disabInfoId == 0 || srvOrganId == null || srvOrganId == 0) {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        ProgramDetailVo programDetail = srvProgramService.getProgramDetailWithAdd(disabInfoId, srvOrganId);
        return Result.success(programDetail);
    }

    /**
     * 添加服务方案接口
     *
     * @param programDetailVo 添加服务方案包装对象
     * @return 返回结果
     */
    @PostMapping("/programAndProgramSetList")
    public Result programAndProgramSetList(ProgramDetailVo programDetailVo) {
        log.info(JSONUtil.toJsonStr(programDetailVo));
        // 参数校验
        if (programDetailVo != null) {
            if (null == programDetailVo.getSrvOrganId() || null == programDetailVo.getDisabInfoId() || programDetailVo.getProgramSetAndProjectVos() == null || programDetailVo.getProgramSetAndProjectVos().size() == 0) {
                return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
            } else {
                List<ProgramSetAndProjectVo> programSetAndProjectVos = programDetailVo.getProgramSetAndProjectVos();
                for (ProgramSetAndProjectVo programSetAndProjectVo : programSetAndProjectVos) {
                    if (programSetAndProjectVo.getIsConf().intValue() == 1 && programSetAndProjectVo.getPlanningFrequency() == null&&programSetAndProjectVo.getPlanningFrequencyType()!= 2) {
                        return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
                    }
                }
            }
        } else {
            return Result.failed(ResultCode.REQUEST_PARAM_ERROR);
        }
        // 添加的话，需要校验其它服务方案是否已终止
        if (programDetailVo.getProgramId() == null) {
            List<ChSrvProgram> srvPrograms = srvProgramService.getProgramSetCountByProgStatus(programDetailVo.getDisabInfoId(), programDetailVo.getSrvOrganId(), "2");
            if (srvPrograms.size() > 0) {
                return Result.failed(ResultCode.REQUEST_PROGRAM_ADD_ERROR);
            }
        }
        // 进行添加或者修改操作
        srvProgramService.addProgramAndProgramSetList(programDetailVo);
        return Result.success();
    }

    /**
     * 终止服务方案
     *
     * @param programId 服务方案id
     * @param endReason 终止原因
     * @return 返回请求结果
     */
    @PutMapping("/endProgram")
    public Result endProgram(Long programId, String endReason) {
        srvProgramService.endProgram(programId, endReason);
        return Result.success();
    }
}
